@@ -20,6 +20,11 @@ int get_last_request(peer_info const& pi)
     return total_seconds(pi.last_request);
 }
 
+int get_download_queue_time(peer_info const& pi)
+{
+    return total_seconds(pi.download_queue_time);
+}
+
 #ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
 str get_country(peer_info const& pi)
 {
@@ -27,6 +32,11 @@ str get_country(peer_info const& pi)
 }
 #endif
 
+tuple get_local_endpoint(peer_info const& pi)
+{
+    return make_tuple(pi.local_endpoint.address().to_string(), pi.local_endpoint.port());
+}
+
 tuple get_ip(peer_info const& pi)
 {
     return make_tuple(pi.ip.address().to_string(), pi.ip.port());
@@ -64,8 +74,13 @@ void bind_peer_info()
         .def_readonly("download_limit", &peer_info::download_limit)
         .add_property("last_request", get_last_request)
         .add_property("last_active", get_last_active)
+        .add_property("download_queue_time", get_download_queue_time)
+        .def_readonly("queue_bytes", &peer_info::queue_bytes)
+        .def_readonly("request_timeout", &peer_info::request_timeout)
         .def_readonly("send_buffer_size", &peer_info::send_buffer_size)
         .def_readonly("used_send_buffer", &peer_info::used_send_buffer)
+        .def_readonly("receive_buffer_size", &peer_info::receive_buffer_size)
+        .def_readonly("used_receive_buffer", &peer_info::used_receive_buffer)
         .def_readonly("num_hashfails", &peer_info::num_hashfails)
 #ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
         .add_property("country", get_country)
@@ -89,7 +104,13 @@ void bind_peer_info()
         .def_readonly("send_quota", &peer_info::send_quota)
         .def_readonly("receive_quota", &peer_info::receive_quota)
         .def_readonly("rtt", &peer_info::rtt)
+        .def_readonly("num_pieces", &peer_info::num_pieces)
+        .def_readonly("download_rate_peak", &peer_info::download_rate_peak)
+        .def_readonly("upload_rate_peak", &peer_info::upload_rate_peak)
         .def_readonly("progress", &peer_info::progress)
+        .def_readonly("progress_ppm", &peer_info::progress_ppm)
+        .def_readonly("estimated_reciprocation_rate", &peer_info::estimated_reciprocation_rate)
+        .add_property("local_endpoint", get_local_endpoint)
         ;
 
     // flags
@@ -104,6 +125,11 @@ void bind_peer_info()
     pi.attr("queued") = (int)peer_info::queued;
     pi.attr("on_parole") = (int)peer_info::on_parole;
     pi.attr("seed") = (int)peer_info::seed;
+    pi.attr("optimistic_unchoke") = (int)peer_info::optimistic_unchoke;
+    pi.attr("snubbed") = (int)peer_info::snubbed;
+    pi.attr("upload_only") = (int)peer_info::upload_only;
+    pi.attr("endgame_mode") = (int)peer_info::endgame_mode;
+    pi.attr("holepunched") = (int)peer_info::holepunched;
 #ifndef TORRENT_DISABLE_ENCRYPTION
     pi.attr("rc4_encrypted") = (int)peer_info::rc4_encrypted;
     pi.attr("plaintext_encrypted") = (int)peer_info::plaintext_encrypted;
@@ -126,6 +152,8 @@ void bind_peer_info()
     pi.attr("bw_torrent") = (int)peer_info::bw_torrent;
     pi.attr("bw_global") = (int)peer_info::bw_global;
 #endif
+    pi.attr("bw_limit") = (int)peer_info::bw_limit;
     pi.attr("bw_network") = (int)peer_info::bw_network;
+    pi.attr("bw_disk") = (int)peer_info::bw_disk;
 }
 
