@@ -30,106 +30,121 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include <iostream>
-#include <fstream>
-#include <iterator>
-#include <iomanip>
-
 #include "libtorrent/entry.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/lazy_entry.hpp"
 #include "libtorrent/magnet_uri.hpp"
-#include <boost/filesystem/operations.hpp>
-
 
 int main(int argc, char* argv[])
 {
 	using namespace libtorrent;
-	using namespace boost::filesystem;
 
-	if (argc != 2)
+	if (argc < 2 || argc > 4)
 	{
-		std::cerr << "usage: dump_torrent torrent-file\n";
+		fputs("usage: dump_torrent torrent-file [total-items-limit] [recursion-limit]\n", stderr);
 		return 1;
 	}
-#if BOOST_VERSION < 103400
-	boost::filesystem::path::default_name_check(boost::filesystem::no_check);
-#endif
+
+	int item_limit = 1000000;
+	int depth_limit = 1000;
+
+	if (argc > 2) item_limit = atoi(argv[2]);
+	if (argc > 3) depth_limit = atoi(argv[3]);
 
 	int size = file_size(argv[1]);
-	if (size > 10 * 1000000)
+	if (size > 40 * 1000000)
 	{
-		std::cerr << "file too big (" << size << "), aborting\n";
+		fprintf(stderr, "file too big (%d), aborting\n", size);
 		return 1;
 	}
 	std::vector<char> buf(size);
-	std::ifstream(argv[1], std::ios_base::binary).read(&buf[0], size);
+	error_code ec;
+	int ret = load_file(argv[1], buf, ec, 40 * 1000000);
+	if (ret != 0)
+	{
+		fprintf(stderr, "failed to load file: %s\n", ec.message().c_str());
+		return 1;
+	}
 	lazy_entry e;
-	int ret = lazy_bdecode(&buf[0], &buf[0] + buf.size(), e);
+	int pos;
+	printf("decoding. recursion limit: %d total item count limit: %d\n"
+		, depth_limit, item_limit);
+	ret = lazy_bdecode(&buf[0], &buf[0] + buf.size(), e, ec, &pos
+		, depth_limit, item_limit);
+
+	printf("\n\n----- raw info -----\n\n%s\n", print_entry(e).c_str());
 
 	if (ret != 0)
 	{
-		std::cerr << "invalid bencoding: " << ret << std::endl;
+		fprintf(stderr, "failed to decode: '%s' at character: %d\n", ec.message().c_str(), pos);
 		return 1;
 	}
 
-	std::cout << "\n\n----- raw info -----\n\n";
-	std::cout << print_entry(e) << std::endl;
-
-	error_code ec;
 	torrent_info t(e, ec);
 	if (ec)
 	{
-		std::cout << ec.message() << std::endl;
+		fprintf(stderr, "%s\n", ec.message().c_str());
 		return 1;
 	}
+	e.clear();
+	std::vector<char>().swap(buf);
 
 	// print info about torrent
-	std::cout << "\n\n----- torrent file info -----\n\n";
-	std::cout << "nodes:\n";
+	printf("\n\n----- torrent file info -----\n\n"
+		"nodes:\n");
+
 	typedef std::vector<std::pair<std::string, int> > node_vec;
 	node_vec const& nodes = t.nodes();
 	for (node_vec::const_iterator i = nodes.begin(), end(nodes.end());
 		i != end; ++i)
 	{
-		std::cout << i->first << ":" << i->second << "\n";
+		printf("%s: %d\n", i->first.c_str(), i->second);
 	}
-	std::cout << "trackers:\n";
+	puts("trackers:\n");
 	for (std::vector<announce_entry>::const_iterator i = t.trackers().begin();
 		i != t.trackers().end(); ++i)
 	{
-		std::cout << i->tier << ": " << i->url << "\n";
+		printf("%2d: %s\n", i->tier, i->url.c_str());
 	}
 
-	std::cout << "number of pieces: " << t.num_pieces() << "\n";
-	std::cout << "piece length: " << t.piece_length() << "\n";
 	char ih[41];
 	to_hex((char const*)&t.info_hash()[0], 20, ih);
-	std::cout << "info hash: " << ih << "\n";
-	std::cout << "comment: " << t.comment() << "\n";
-	std::cout << "created by: " << t.creator() << "\n";
-	std::cout << "magnet link: " << make_magnet_uri(t) << "\n";
-	std::cout << "name: " << t.name() << "\n";
-	std::cout << "files:\n";
+	printf("number of pieces: %d\n"
+		"piece length: %d\n"
+		"info hash: %s\n"
+		"comment: %s\n"
+		"created by: %s\n"
+		"magnet link: %s\n"
+		"name: %s\n"
+		"number of files: %d\n"
+		"files:\n"
+		, t.num_pieces()
+		, t.piece_length()
+		, ih
+		, t.comment().c_str()
+		, t.creator().c_str()
+		, make_magnet_uri(t).c_str()
+		, t.name().c_str()
+		, t.num_files());
 	int index = 0;
 	for (torrent_info::file_iterator i = t.begin_files();
 		i != t.end_files(); ++i, ++index)
 	{
 		int first = t.map_file(index, 0, 0).piece;
-		int last = t.map_file(index, (std::max)(i->size-1, size_type(0)), 0).piece;
-		std::cout << "  " << std::setw(11) << i->size
-			<< " "
-			<< (i->pad_file?'p':'-')
-			<< (i->executable_attribute?'x':'-')
-			<< (i->hidden_attribute?'h':'-')
-			<< (i->symlink_attribute?'l':'-')
-			<< " "
-			<< "[ " << std::setw(4) << first << ", " << std::setw(4) << last << " ]\t"
-			<< i->path.string() ;
-		if (i->symlink_attribute)
-			std::cout << " -> " << i->symlink_path.string();
-		std::cout << std::endl;
+		int last = t.map_file(index, (std::max)(size_type(i->size)-1, size_type(0)), 0).piece;
+		printf("  %11"PRId64" %c%c%c%c [ %4d, %4d ] %7u %s %s %s%s\n"
+			, i->size
+			, (i->pad_file?'p':'-')
+			, (i->executable_attribute?'x':'-')
+			, (i->hidden_attribute?'h':'-')
+			, (i->symlink_attribute?'l':'-')
+			, first, last
+			, boost::uint32_t(t.files().mtime(*i))
+			, t.files().hash(*i) != sha1_hash(0) ? to_hex(t.files().hash(*i).to_string()).c_str() : ""
+			, t.files().file_path(*i).c_str()
+			, i->symlink_attribute ? "-> ": ""
+			, i->symlink_attribute && i->symlink_index != -1 ? t.files().symlink(*i).c_str() : "");
 	}
 
 	return 0;
