@@ -34,6 +34,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_PEER_INFO_HPP_INCLUDED
 
 #include "libtorrent/socket.hpp"
+#include "libtorrent/deadline_timer.hpp"
 #include "libtorrent/peer_id.hpp"
 #include "libtorrent/size_type.hpp"
 #include "libtorrent/config.hpp"
@@ -59,7 +60,8 @@ namespace libtorrent
 			optimistic_unchoke = 0x800,
 			snubbed = 0x1000,
 			upload_only = 0x2000,
-			endgame_mode = 0x4000
+			endgame_mode = 0x4000,
+			holepunched = 0x8000
 #ifndef TORRENT_DISABLE_ENCRYPTION
 			, rc4_encrypted = 0x100000,
 			plaintext_encrypted = 0x200000
@@ -85,8 +87,9 @@ namespace libtorrent
 		// bw_network: the channel is waiting for an async write
 		//   for read operation to complete
 		// bw_disk: the peer is waiting for the disk io thread
-		//   to catch up
-		enum bw_state { bw_idle, bw_limit, bw_network, bw_disk };
+		// this is a bitmask, a peer can wait for network and
+		// disk at the same time!
+		enum bw_state { bw_idle = 0, bw_limit = 1, bw_network = 2, bw_disk = 4 };
 #ifndef TORRENT_NO_DEPRECATE
 		enum bw_state_deprecated { bw_torrent = bw_limit, bw_global = bw_limit };
 #endif
@@ -153,6 +156,16 @@ namespace libtorrent
 		// that we haven't got a response
 		// for yet
 		int download_queue_length;
+		
+		// the number of block requests that have
+		// timed out, and are still in the download
+		// queue
+		int timed_out_requests;
+
+		// the number of busy requests in the download
+		// queue. A budy request is a request for a block
+		// we've also requested from a different peer
+		int busy_requests;
 
 		// the number of request messages
 		// waiting to be sent inside the send buffer
@@ -186,7 +199,8 @@ namespace libtorrent
 		{
 			standard_bittorrent = 0,
 			web_seed = 1,
-			http_seed = 2
+			http_seed = 2,
+			bittorrent_utp = 3
 		};
 		int connection_type;
 		
@@ -214,6 +228,10 @@ namespace libtorrent
 		// the peers progress
 		float progress; // [0, 1]
 		int progress_ppm; // [0, 1000000]
+
+		int estimated_reciprocation_rate;
+
+		tcp::endpoint local_endpoint;
 	};
 
 	struct TORRENT_EXPORT peer_list_entry
