@@ -125,7 +125,7 @@ bool node_impl::verify_token(std::string const& token, char const* info_hash
 	h1.update((char*)info_hash, sha1_hash::size);
 	
 	sha1_hash h = h1.final();
-	if (std::equal(token.begin(), token.end(), (signed char*)&h[0]))
+	if (std::equal(token.begin(), token.end(), (char*)&h[0]))
 		return true;
 		
 	hasher h2;
@@ -133,7 +133,7 @@ bool node_impl::verify_token(std::string const& token, char const* info_hash
 	h2.update((char*)&m_secret[1], sizeof(m_secret[1]));
 	h2.update((char*)info_hash, sha1_hash::size);
 	h = h2.final();
-	if (std::equal(token.begin(), token.end(), (signed char*)&h[0]))
+	if (std::equal(token.begin(), token.end(), (char*)&h[0]))
 		return true;
 	return false;
 }
@@ -151,7 +151,8 @@ std::string node_impl::generate_token(udp::endpoint const& addr, char const* inf
 	h.update(info_hash, sha1_hash::size);
 
 	sha1_hash hash = h.final();
-	std::copy(hash.begin(), hash.begin() + 4, (signed char*)&token[0]);
+	std::copy(hash.begin(), hash.begin() + 4, (char*)&token[0]);
+	TORRENT_ASSERT(std::equal(token.begin(), token.end(), (char*)&hash[0]));
 	return token;
 }
 
@@ -573,6 +574,10 @@ bool verify_message(lazy_entry const* msg, key_desc_t const desc[], lazy_entry c
 		else if (k.flags & key_desc_t::last_child)
 		{
 			TORRENT_ASSERT(stack_ptr > 0);
+			// this can happen if the specification passed
+			// in is unbalanced. i.e. contain more last_child
+			// nodes than parse_children
+			if (stack_ptr == 0) return false;
 			--stack_ptr;
 			msg = stack[stack_ptr];
 		}
