@@ -79,6 +79,7 @@ udp_socket::udp_socket(asio::io_service& ios
 	, m_resolver(ios)
 	, m_queue_packets(false)
 	, m_tunnel_packets(false)
+	, m_force_proxy(false)
 	, m_abort(false)
 	, m_outstanding_ops(0)
 {
@@ -151,8 +152,14 @@ void udp_socket::send_hostname(char const* hostname, int port
 	}
 
 	// this function is only supported when we're using a proxy
-	TORRENT_ASSERT(m_queue_packets);
-	if (!m_queue_packets) return;
+	if (!m_queue_packets)
+	{
+		address target = address::from_string(hostname, ec);
+		if (!ec) send(udp::endpoint(target, port), p, len, ec, 0);
+		return;
+	}
+
+	if (m_queue.size() > 1000) return;
 
 	m_queue.push_back(queued_packet());
 	queued_packet& qp = m_queue.back();
@@ -199,6 +206,8 @@ void udp_socket::send(udp::endpoint const& ep, char const* p, int len
 
 		if (m_queue_packets)
 		{
+			if (m_queue.size() > 1000) return;
+
 			m_queue.push_back(queued_packet());
 			queued_packet& qp = m_queue.back();
 			qp.ep = ep;
@@ -209,6 +218,8 @@ void udp_socket::send(udp::endpoint const& ep, char const* p, int len
 		}
 	}
 
+	if (m_force_proxy) return;
+
 #if TORRENT_USE_IPV6
 	if (ep.address().is_v4() && m_ipv4_sock.is_open())
 #endif
@@ -365,7 +376,7 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 			if (m_tunnel_packets)
 			{
 				// if the source IP doesn't match the proxy's, ignore the packet
-				if (m_v6_ep == m_proxy_addr)
+				if (m_v6_ep == m_udp_proxy_addr)
 					unwrap(e, m_v6_buf, bytes_transferred);
 			}
 			else
@@ -399,7 +410,7 @@ void udp_socket::on_read(udp::socket* s, error_code const& e, std::size_t bytes_
 			if (m_tunnel_packets)
 			{
 				// if the source IP doesn't match the proxy's, ignore the packet
-				if (m_v4_ep == m_proxy_addr)
+				if (m_v4_ep == m_udp_proxy_addr)
 					unwrap(e, m_v4_buf, bytes_transferred);
 			}
 			else
@@ -448,12 +459,12 @@ void udp_socket::wrap(udp::endpoint const& ep, char const* p, int len, error_cod
 	iovec[1] = asio::const_buffer(p, len);
 
 #if TORRENT_USE_IPV6
-	if (m_proxy_addr.address().is_v4() && m_ipv4_sock.is_open())
+	if (m_udp_proxy_addr.address().is_v4() && m_ipv4_sock.is_open())
 #endif
-		m_ipv4_sock.send_to(iovec, m_proxy_addr, 0, ec);
+		m_ipv4_sock.send_to(iovec, m_udp_proxy_addr, 0, ec);
 #if TORRENT_USE_IPV6
 	else
-		m_ipv6_sock.send_to(iovec, m_proxy_addr, 0, ec);
+		m_ipv6_sock.send_to(iovec, m_udp_proxy_addr, 0, ec);
 #endif
 }
 
@@ -479,12 +490,12 @@ void udp_socket::wrap(char const* hostname, int port, char const* p, int len, er
 	iovec[1] = asio::const_buffer(p, len);
 
 #if TORRENT_USE_IPV6
-	if (m_proxy_addr.address().is_v4() && m_ipv4_sock.is_open())
+	if (m_udp_proxy_addr.address().is_v4() && m_ipv4_sock.is_open())
 #endif
-		m_ipv4_sock.send_to(iovec, m_proxy_addr, 0, ec);
+		m_ipv4_sock.send_to(iovec, m_udp_proxy_addr, 0, ec);
 #if TORRENT_USE_IPV6
 	else
-		m_ipv6_sock.send_to(iovec, m_proxy_addr, 0, ec);
+		m_ipv6_sock.send_to(iovec, m_udp_proxy_addr, 0, ec);
 #endif
 }
 
@@ -790,6 +801,9 @@ void udp_socket::on_name_lookup(error_code const& e, tcp::resolver::iterator i)
 		TORRENT_TRY {
 			if (m_callback) m_callback(e, udp::endpoint(), 0, 0);
 		} TORRENT_CATCH (std::exception&) {}
+
+		drain_queue();
+
 		return;
 	}
 
@@ -832,6 +846,8 @@ void udp_socket::on_timeout()
 		+ m_outstanding_connect_queue
 		+ m_outstanding_socks);
 
+	m_queue_packets = false;
+
 	if (m_abort)
 	{
 		maybe_clear_callback();
@@ -1013,7 +1029,11 @@ void udp_socket::handshake1(error_code const& e)
 		return;
 	}
 	CHECK_MAGIC;
-	if (e) return;
+	if (e)
+	{
+		drain_queue();
+		return;
+	}
 
 	TORRENT_ASSERT(is_single_thread());
 
@@ -1052,7 +1072,11 @@ void udp_socket::handshake2(error_code const& e)
 	}
 	CHECK_MAGIC;
 
-	if (e) return;
+	if (e)
+	{
+		drain_queue();
+		return;
+	}
 
 	using namespace libtorrent::detail;
 
@@ -1062,7 +1086,13 @@ void udp_socket::handshake2(error_code const& e)
 	int version = read_uint8(p);
 	int method = read_uint8(p);
 
-	if (version < 5) return;
+	if (version < 5)
+	{
+		error_code ec;
+		m_socks5_sock.close(ec);
+		drain_queue();
+		return;
+	}
 
 	if (method == 0)
 	{
@@ -1074,6 +1104,7 @@ void udp_socket::handshake2(error_code const& e)
 		{
 			error_code ec;
 			m_socks5_sock.close(ec);
+			drain_queue();
 			return;
 		}
 
@@ -1097,6 +1128,7 @@ void udp_socket::handshake2(error_code const& e)
 	}
 	else
 	{
+		drain_queue();
 		error_code ec;
 		m_socks5_sock.close(ec);
 		return;
@@ -1126,7 +1158,11 @@ void udp_socket::handshake3(error_code const& e)
 		return;
 	}
 	CHECK_MAGIC;
-	if (e) return;
+	if (e)
+	{
+		drain_queue();
+		return;
+	}
 
 	TORRENT_ASSERT(is_single_thread());
 
@@ -1164,7 +1200,11 @@ void udp_socket::handshake4(error_code const& e)
 		return;
 	}
 	CHECK_MAGIC;
-	if (e) return;
+	if (e)
+	{
+		drain_queue();
+		return;
+	}
 
 	TORRENT_ASSERT(is_single_thread());
 
@@ -1174,8 +1214,11 @@ void udp_socket::handshake4(error_code const& e)
 	int version = read_uint8(p);
 	int status = read_uint8(p);
 
-	if (version != 1) return;
-	if (status != 0) return;
+	if (version != 1 || status != 0)
+	{
+		drain_queue();
+		return;
+	}
 
 	socks_forward_udp(/*l*/);
 }
@@ -1191,19 +1234,9 @@ void udp_socket::socks_forward_udp()
 	write_uint8(3, p); // UDP ASSOCIATE command
 	write_uint8(0, p); // reserved
 	error_code ec;
-	tcp::endpoint local = m_socks5_sock.local_endpoint(ec);
-	write_uint8(local.address().is_v4() ? 1 : 4, p); // ATYP IPv4
-	detail::write_address(local.address(), p);
-	int port = 0;
-#if TORRENT_USE_IPV6
-	if (local.address().is_v4())
-#endif
-		port = m_ipv4_sock.local_endpoint(ec).port();
-#if TORRENT_USE_IPV6
-	else
-		port = m_ipv6_sock.local_endpoint(ec).port();
-#endif
-	detail::write_uint16(port , p);
+	write_uint8(1, p); // ATYP = IPv4
+	write_uint32(0, p); // 0.0.0.0
+	write_uint16(0, p); // :0
 	TORRENT_ASSERT_VAL(p - m_tmp_buf < int(sizeof(m_tmp_buf)), (p - m_tmp_buf));
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("udp_socket::connect1");
@@ -1239,7 +1272,11 @@ void udp_socket::connect1(error_code const& e)
 		return;
 	}
 	CHECK_MAGIC;
-	if (e) return;
+	if (e)
+	{
+		drain_queue();
+		return;
+	}
 
 	TORRENT_ASSERT(is_single_thread());
 
@@ -1280,7 +1317,7 @@ void udp_socket::connect2(error_code const& e)
 	CHECK_MAGIC;
 	if (e)
 	{
-		m_queue.clear();
+		drain_queue();
 		return;
 	}
 
@@ -1296,42 +1333,25 @@ void udp_socket::connect2(error_code const& e)
 
 	if (version != 5 || status != 0)
 	{
-		m_queue.clear();
+		drain_queue();
 		return;
 	}
 
 	if (atyp == 1)
 	{
-		m_proxy_addr.address(address_v4(read_uint32(p)));
-		m_proxy_addr.port(read_uint16(p));
+		m_udp_proxy_addr.address(address_v4(read_uint32(p)));
+		m_udp_proxy_addr.port(read_uint16(p));
 	}
 	else
 	{
 		// in this case we need to read more data from the socket
 		TORRENT_ASSERT(false && "not implemented yet!");
-		m_queue.clear();
+		drain_queue();
 		return;
 	}
 	
 	m_tunnel_packets = true;
-	m_queue_packets = false;
-
-	// forward all packets that were put in the queue
-	while (!m_queue.empty())
-	{
-		queued_packet const& p = m_queue.front();
-		error_code ec;
-		if (p.hostname)
-		{
-			udp_socket::send_hostname(p.hostname, p.ep.port(), &p.buf[0], p.buf.size(), ec);
-			free(p.hostname);
-		}
-		else
-		{
-			udp_socket::send(p.ep, &p.buf[0], p.buf.size(), ec, p.flags);
-		}
-		m_queue.pop_front();
-	}
+	drain_queue();
 
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("udp_socket::hung_up");
@@ -1375,6 +1395,28 @@ void udp_socket::hung_up(error_code const& e)
 	set_proxy_settings(m_proxy_settings);
 }
 
+void udp_socket::drain_queue()
+{
+	m_queue_packets = false;
+
+	// forward all packets that were put in the queue
+	while (!m_queue.empty())
+	{
+		queued_packet const& p = m_queue.front();
+		error_code ec;
+		if (p.hostname)
+		{
+			udp_socket::send_hostname(p.hostname, p.ep.port(), &p.buf[0], p.buf.size(), ec);
+			free(p.hostname);
+		}
+		else if (!m_force_proxy) // block incoming packets that aren't coming via the proxy
+		{
+			udp_socket::send(p.ep, &p.buf[0], p.buf.size(), ec, p.flags);
+		}
+		m_queue.pop_front();
+	}
+}
+
 rate_limited_udp_socket::rate_limited_udp_socket(io_service& ios
 	, callback_t const& c
 	, callback2_t const& c2
