@@ -71,7 +71,7 @@ upnp::upnp(io_service& ios, connection_queue& cc
 	, m_log_callback(lcb)
 	, m_retry_count(0)
 	, m_io_service(ios)
-	, m_socket(ios, udp::endpoint(address_v4::from_string("239.255.255.250", ec), 1900)
+	, m_socket(udp::endpoint(address_v4::from_string("239.255.255.250", ec), 1900)
 		, boost::bind(&upnp::on_reply, self(), _1, _2, _3))
 	, m_broadcast_timer(ios)
 	, m_refresh_timer(ios)
@@ -82,6 +82,9 @@ upnp::upnp(io_service& ios, connection_queue& cc
 {
 	TORRENT_ASSERT(cb);
 
+	error_code ec;
+	m_socket.open(ios, ec);
+
 	if (state)
 	{
 		upnp_state_t* s = (upnp_state_t*)state;
@@ -145,7 +148,8 @@ void upnp::discover_device_impl(mutex::scoped_lock& l)
 	if (ec)
 	{
 		char msg[200];
-		snprintf(msg, sizeof(msg), "broadcast failed: %s. Aborting.", ec.message().c_str());
+		snprintf(msg, sizeof(msg), "broadcast failed: %s. Aborting."
+			, convert_from_native(ec.message()).c_str());
 		log(msg, l);
 		disable(ec, l);
 		return;
@@ -352,7 +356,7 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 		{
 			char msg[200];
 			snprintf(msg, sizeof(msg), "when receiving response from: %s: %s"
-				, print_endpoint(from).c_str(), ec.message().c_str());
+				, print_endpoint(from).c_str(), convert_from_native(ec.message()).c_str());
 			log(msg, l);
 		}
 		else
@@ -386,7 +390,7 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 			{
 				char msg[200];
 				snprintf(msg, sizeof(msg), "when receiving response from: %s: %s"
-					, print_endpoint(from).c_str(), ec.message().c_str());
+					, print_endpoint(from).c_str(), convert_from_native(ec.message()).c_str());
 				log(msg, l);
 			}
 			else
@@ -476,7 +480,7 @@ void upnp::on_reply(udp::endpoint const& from, char* buffer
 		{
 			char msg[200];
 			snprintf(msg, sizeof(msg), "invalid URL %s from %s: %s"
-				, d.url.c_str(), print_endpoint(from).c_str(), ec.message().c_str());
+				, d.url.c_str(), print_endpoint(from).c_str(), convert_from_native(ec.message()).c_str());
 			log(msg, l);
 			return;
 		}
@@ -853,7 +857,7 @@ void upnp::on_upnp_xml(error_code const& e
 	{
 		char msg[200];
 		snprintf(msg, sizeof(msg), "error while fetching control url from: %s: %s"
-			, d.url.c_str(), e.message().c_str());
+			, d.url.c_str(), convert_from_native(e.message()).c_str());
 		log(msg, l);
 		d.disabled = true;
 		return;
@@ -873,7 +877,7 @@ void upnp::on_upnp_xml(error_code const& e
 	{
 		char msg[200];
 		snprintf(msg, sizeof(msg), "error while fetching control url from: %s: %s"
-			, d.url.c_str(), p.message().c_str());
+			, d.url.c_str(), convert_from_native(p.message()).c_str());
 		log(msg, l);
 		d.disabled = true;
 		return;
@@ -947,7 +951,7 @@ void upnp::on_upnp_xml(error_code const& e
 	{
 		char msg[200];
 		snprintf(msg, sizeof(msg), "failed to parse URL '%s': %s"
-			, d.control_url.c_str(), ec.message().c_str());
+			, d.control_url.c_str(), convert_from_native(ec.message()).c_str());
 		log(msg, l);
 		d.disabled = true;
 		return;
@@ -1145,7 +1149,8 @@ void upnp::on_upnp_get_ip_address_response(error_code const& e
 	if (e && e != asio::error::eof)
 	{
 		char msg[200];
-		snprintf(msg, sizeof(msg), "error while getting external IP address: %s", e.message().c_str());
+		snprintf(msg, sizeof(msg), "error while getting external IP address: %s"
+			, convert_from_native(e.message()).c_str());
 		log(msg, l);
 		if (num_mappings() > 0) update_map(d, 0, l);
 		return;
@@ -1161,7 +1166,8 @@ void upnp::on_upnp_get_ip_address_response(error_code const& e
 	if (p.status_code() != 200)
 	{
 		char msg[200];
-		snprintf(msg, sizeof(msg), "error while getting external IP address: %s", p.message().c_str());
+		snprintf(msg, sizeof(msg), "error while getting external IP address: %s"
+			, convert_from_native(p.message()).c_str());
 		log(msg, l);
 		if (num_mappings() > 0) update_map(d, 0, l);
 		return;
@@ -1222,7 +1228,7 @@ void upnp::on_upnp_map_response(error_code const& e
 	{
 		char msg[200];
 		snprintf(msg, sizeof(msg), "error while adding port map: %s"
-			, e.message().c_str());
+			, convert_from_native(e.message()).c_str());
 		log(msg, l);
 		d.disabled = true;
 		return;
@@ -1383,7 +1389,8 @@ void upnp::on_upnp_unmap_response(error_code const& e
 	if (e && e != asio::error::eof)
 	{
 		char msg[200];
-		snprintf(msg, sizeof(msg), "error while deleting portmap: %s", e.message().c_str());
+		snprintf(msg, sizeof(msg), "error while deleting portmap: %s"
+			, convert_from_native(e.message()).c_str());
 		log(msg, l);
 	}
 	else if (!p.header_finished())
@@ -1393,7 +1400,8 @@ void upnp::on_upnp_unmap_response(error_code const& e
 	else if (p.status_code() != 200)
 	{
 		char msg[200];
-		snprintf(msg, sizeof(msg), "error while deleting portmap: %s", p.message().c_str());
+		snprintf(msg, sizeof(msg), "error while deleting portmap: %s"
+			, convert_from_native(p.message()).c_str());
 		log(msg, l);
 	}
 	else
